@@ -30,7 +30,6 @@
 # (3) Parse the usage information (default usage file name: current file's name with -usage appended).
 # (4) Parse the command line using the usage information.
 
-
 export OPENCOARRAYS_SRC_DIR="${OPENCOARRAYS_SRC_DIR:-${PWD%prerequisites*}}"
 export __usage=${OPENCOARRAYS_SRC_DIR}/prerequisites/build.sh-usage
 if [[ ! -f "${OPENCOARRAYS_SRC_DIR}/src/libcaf.h" ]]; then
@@ -107,10 +106,12 @@ info "-n (--no-color):         ${arg_n} "
 info "-p (--package):          ${arg_p}"
 info "-P (--print-path):       ${arg_P} "
 info "-t (--with-tau):         ${arg_t} "
+info "-u (--from-url):         ${arg_u} "
 info "-U (--print-url):        ${arg_U} "
 info "-v (--version):          ${arg_v} "
 info "-V (--print-version):    ${arg_V} "
 info "-y (--yes-to-all):       ${arg_y} "
+info "-z (--disable-bootstrap):${arg_z} "
 }
 
 if [[ -z "${arg_B}" ]]; then
@@ -141,6 +142,21 @@ fi
 source "${OPENCOARRAYS_SRC_DIR:-}/prerequisites/build-functions/download_if_necessary.sh"
 download_if_necessary
 
+# Exit if -o or --only-download was specified when invoking this script
+if [[ ${arg_o:-} == "${__flag_present}" ]]; then
+   info "No installation to perform: -o or --only-download specified.  Exiting."
+   exit 0
+fi
+
+# If -z or --disable-bootstrap was specified, disable bootstrap configure & build
+if [[ ${arg_z:-} == "${__flag_present}" ]]; then
+   export bootstrap_configure="--disable-bootstrap"
+   export bootstrap_build=""
+else
+   export bootstrap_configure=""
+   export bootstrap_build="bootstrap"
+fi
+
 if [[ -z "${arg_B}" ]]; then
   # shellcheck source=./build-functions/unpack_if_necessary.sh
   source "${OPENCOARRAYS_SRC_DIR:-}/prerequisites/build-functions/unpack_if_necessary.sh"
@@ -149,7 +165,7 @@ if [[ -z "${arg_B}" ]]; then
   # shellcheck source=./build-functions/set_compilers.sh
   source "${OPENCOARRAYS_SRC_DIR:-}/prerequisites/build-functions/set_compilers.sh"
   set_compilers
- 
+
   # shellcheck source=./build-functions/build_and_install.sh
   source "${OPENCOARRAYS_SRC_DIR:-}/prerequisites/build-functions/build_and_install.sh"
   build_and_install
